indexing

   description: "Standard C dynamic memory. Memory should be at least 1 byte%
   %large. 0 size memory is not supported."

   author: "Berend de Boer"
   date: "$Date: Thu, 14 Sep 2000 17:06:24 +0200 $"
   revision: "$Revision: 1.3 $"

class 

   STDC_DYNAMIC_MEMORY

inherit

   STDC_BASE

   MEMORY
      redefine
         dispose
      end
   
   CAPI_STDLIB

   CAPI_STRING


creation
   
   allocate,
   allocate_and_clear

   
feature -- public features

   allocate (a_size: INTEGER) is
         -- allocate memory of `a_size' bytes
      require
         valid_size: a_size > 0
         not_allocated: not is_allocated
      do
         if ptr /= default_pointer then
            deallocate
         end
         ptr := posix_malloc (a_size)
         if ptr = default_pointer then
            raise_posix_error
         end
         size := a_size
      ensure
         successfull_allocation: is_allocated
      end
   
   allocate_and_clear (a_size: INTEGER) is
         -- allocate memory of `a_size' bytes, make sure it's zeroed out
      require
         valid_size: a_size > 0
         not_allocated: not is_allocated
      do
         if ptr /= default_pointer then
            deallocate
         end
         ptr := posix_calloc (a_size, 1)
         if ptr = default_pointer then
            raise_posix_error
         end
         size := a_size
      ensure
         successfull_allocation: is_allocated
      end

   deallocate is
         -- free the allocated memory now, don't wait for garbage collector.
      require
         not_deallocated: is_allocated
      do
         posix_free  (ptr)
         ptr := default_pointer
         size := 0
      ensure
         now_deallocated: not is_allocated
      end

   resize, realloc (new_size: INTEGER) is
         -- resize memory to `new_size' bytes. Expanded memory is not
         -- guaranteed to be zeroed out.
      require
         valid_size: new_size > 0
         allocated: is_allocated
      do
         ptr := posix_realloc (ptr, new_size)
         if ptr = default_pointer then
            raise_posix_error
         end
         size := new_size
      ensure
         successfull_allocation: is_allocated
      end


feature -- copy data from somewhere
   
   copy_from (source: POINTER; a_size: INTEGER) is
         -- Copy data from `source', memory may not overlap
      require
         data_should_fit: a_size <= size
      do
         posix_memcpy (ptr, source, a_size)
      end

                
feature -- set/get bytes (8-bit data)
   
   peek_byte (index: INTEGER): INTEGER is
         -- consider memory an array of 8 bit values.
      require
         valid_index: index >= 0 and index < size
      do
         Result := posix_peek_byte (ptr, index)
      ensure
         possible_values: Result >= 0 and Result < 256         
      end
   
   poke_byte (index, value: INTEGER) is
      require
         valid_index: index >= 0 and index < size
         valid_value: value >= 0 and value < 256
      do
         posix_poke_byte (ptr, index, value)
      end


feature -- set/get integers (32-bit data)
   
   peek_integer (index: INTEGER): INTEGER is
         -- consider memory an array of 32 bit values.
      require
         valid_index: index >= 0 and index < (size // 4)
      do
         Result := posix_peek_integer (ptr, index)
      end
   

feature -- queries
   
   is_allocated: BOOLEAN is
      do
         Result := ptr /= default_pointer
      end

   
feature -- state   
   
   ptr: POINTER
         -- the actual pointer
   
   size: INTEGER
         -- in number of bytes


feature

   dispose is
      do
         if is_allocated then
            deallocate
         end
      end


feature {NONE} -- POSIX c interface
   
   posix_peek_byte (p: POINTER; index: INTEGER): INTEGER is
         -- read byte at position `index'
      require
         valid_memory: p /= default_pointer
      external "C"
      end

   posix_peek_integer (p: POINTER; index: INTEGER): INTEGER is
         -- read integer at position `index'
      require
         valid_memory: p /= default_pointer
      external "C"
      end

   posix_poke_byte (p: POINTER; index: INTEGER; value: INTEGER) is
         -- read byte at position `index'
      require
         valid_memory: p /= default_pointer
      external "C"
      end
   
  
invariant   
   
   valid_size: 
     size >= 0
   size_and_ptr_relation: 
     (size = 0 implies not is_allocated) and
     (size > 0 implies is_allocated)

   
end -- class STDC_DYNAMIC_MEMORY
